@@ -58,7 +58,7 @@ def load_default_parameters():
 
 
 def get_parameter_descriptions():
-    """Get the description for paramters"""
+    """Get the description for parameters"""
     with open(os.path.join(frank_path, 'parameter_descriptions.json')) as f:
         param_descrip = json.load(f)
     return param_descrip
@@ -159,6 +159,10 @@ def parse_parameters(*args):
             except TypeError:
                 raise err
 
+    if model['plotting']['stretch'] not in ["power", "asinh"]:
+        err = ValueError("stretch should be 'power'  or 'asinh'")
+        raise err
+
     if model['modify_data']['baseline_range'] is not None:
         err = ValueError("baseline_range should be 'null' (None)"
                          " or a list specifying the low and high"
@@ -236,9 +240,9 @@ def alter_data(u, v, vis, weights, geom, model):
     to the modification operations specified in model
     """
 
-    if model['modify_data']['normalization_wle'] is not None:
+    if model['modify_data']['norm_wle'] is not None:
         u, v = utilities.normalize_uv(
-            u, v, model['modify_data']['normalization_wle'])
+            u, v, model['modify_data']['norm_wle'])
 
     if model['modify_data']['baseline_range']:
         u, v, vis, weights = \
@@ -248,7 +252,8 @@ def alter_data(u, v, vis, weights, geom, model):
 
     if model['modify_data']['correct_weights']:
         up, vp = geom.deproject(u,v)
-        weights = utilities.estimate_weights(up, vp, vis, use_median=True)
+        weights = utilities.estimate_weights(up, vp, vis,
+                                            use_median=model['modify_data']['use_median_weight'])
 
     return u, v, vis, weights
 
@@ -303,20 +308,28 @@ def determine_geometry(u, v, vis, weights, model):
         else:
             guess = None
 
+        if model['geometry']['fit_inc_pa']:
+            inc_pa = None
+        else:
+            inc_pa = (model['geometry']['inc'],
+                      model['geometry']['pa'])
+
         if model['geometry']['fit_phase_offset']:
+            phase_centre = None
+        else:
             phase_centre = (model['geometry']['dra'],
                             model['geometry']['ddec'])
-        else:
-            phase_centre = None
 
 
         if model['geometry']['type'] == 'gaussian':
             geom = geometry.FitGeometryGaussian(
+                inc_pa=inc_pa,
                 phase_centre=phase_centre, guess=guess,
             )
         else:
             geom = geometry.FitGeometryFourierBessel(
                 model['hyperparameters']['rout'], N=20,
+                inc_pa=inc_pa,
                 phase_centre=phase_centre, guess=guess
             )
 
@@ -453,20 +466,27 @@ def run_multiple_fits(u, v, vis, weights, geom, model):
 
             logging.info('  Running fit for alpha = {}, wsmooth = {}'.format(alphas[ii], ws[jj]))
 
-            sol, _ = perform_fit(u, v, vis, weights, geom, this_model)
+            sol, iteration_diagnostics = perform_fit(u, v, vis, weights, geom, this_model)
             sols.append(sol)
 
             # Save the fit for the current choice of hyperparameter values
-            output_results(u, v, vis, weights, sol, geom, this_model)
+            output_results(u, v, vis, weights, sol, geom, this_model,
+                           iteration_diagnostics=iteration_diagnostics)
 
-    multifit_fig, multifit_axes = make_figs.make_multifit_fig(u, v, vis, weights, sols,
-                                                           model['plotting']['bin_widths'],
-                                                           ['alpha', 'wsmooth'],
-                                                           [alphas, ws],
-                                                           model['plotting']['distance'],
-                                                           model['plotting']['force_style'],
-                                                           model['input_output']['save_prefix'],
-                                                           )
+    if len(alphas) in [1,2] and len(ws) in [1,2]:
+        multifit_fig, multifit_axes = make_figs.make_multifit_fig(u=u, v=v, vis=vis,
+                                                               weights=weights, sols=sols,
+                                                               bin_widths=model['plotting']['bin_widths'],
+                                                               varied_pars=['alpha', 'wsmooth'],
+                                                               varied_vals=[alphas, ws],
+                                                               dist=model['plotting']['distance'],
+                                                               force_style=model['plotting']['force_style'],
+                                                               save_prefix=model['input_output']['save_prefix'],
+                                                               )
+    else:
+        logging.info('The multifit figure requires alpha and wsmooth to be lists of length <= 2.'
+                     'Your lists are length {} and {} --> The multifit figure will not be made.'.format(len(alphas), len(ws)))
+        multifit_fig, multifit_axes = None, None
 
     return multifit_fig, multifit_axes
 
@@ -504,107 +524,143 @@ def output_results(u, v, vis, weights, sol, geom, model, iteration_diagnostics=N
         Axes for each of the produced figures
     """
 
-    logging.info('  Plotting results')
+    if any([model['input_output']['save_solution'],
+            model['input_output']['save_profile_fit'],
+            model['input_output']['save_vis_fit'],
+            model['input_output']['save_uvtables']
+            ]):
+            
+        io.save_fit(u, v, vis, weights, sol,
+                    model['input_output']['save_prefix'],
+                    model['input_output']['save_solution'],
+                    model['input_output']['save_profile_fit'],
+                    model['input_output']['save_vis_fit'],
+                    model['input_output']['save_uvtables'],
+                    model['input_output']['iteration_diag'],
+                    iteration_diagnostics,
+                    model['input_output']['format']
+                    )
+
 
     figs, axes = [], []
 
-    if model['plotting']['deprojec_plot']:
-        deproj_fig, deproj_axes = make_figs.make_deprojection_fig(u, v, vis, geom,
-                                                         model['plotting']['force_style'],
-                                                         model['input_output']['save_prefix']
-                                                         )
+    if any([model['plotting']['deprojec_plot'],
+            model['plotting']['quick_plot'],
+            model['plotting']['full_plot'],
+            model['plotting']['diag_plot'],
+            model['analysis']['compare_profile']
+            ]):
 
-        figs.append(deproj_fig)
-        axes.append(deproj_axes)
+        logging.info('  Plotting results')
 
-    if model['plotting']['quick_plot']:
-        quick_fig, quick_axes = make_figs.make_quick_fig(u, v, vis, weights, sol,
-                                                         model['plotting']['bin_widths'],
-                                                         model['plotting']['distance'],
-                                                         model['plotting']['force_style'],
-                                                         model['input_output']['save_prefix']
-                                                         )
+        priors = {'alpha': model['hyperparameters']['alpha'],\
+                  'wsmooth': model['hyperparameters']['wsmooth'],\
+                  'Rmax': model['hyperparameters']['rout'],\
+                  'N': model['hyperparameters']['n'],\
+                  'p0': model['hyperparameters']['p0']
+                  }
 
-        figs.append(quick_fig)
-        axes.append(quick_axes)
+        if model['plotting']['deprojec_plot']:
+            deproj_fig, deproj_axes = make_figs.make_deprojection_fig(u=u, v=v,
+                                                             vis=vis, weights=weights,
+                                                             geom=geom,
+                                                             bin_widths=model['plotting']['bin_widths'],
+                                                             force_style=model['plotting']['force_style'],
+                                                             save_prefix=model['input_output']['save_prefix']
+                                                             )
 
-    if model['plotting']['full_plot']:
-        full_fig, full_axes = make_figs.make_full_fig(u, v, vis, weights, sol,
-                                                      model['plotting']['bin_widths'],
-                                                      model['hyperparameters']['alpha'],
-                                                      model['hyperparameters']['wsmooth'],
-                                                      model['plotting']['gamma'],
-                                                      model['plotting']['distance'],
-                                                      model['plotting']['force_style'],
-                                                      model['input_output']['save_prefix']
-                                                      )
+            figs.append(deproj_fig)
+            axes.append(deproj_axes)
 
-        figs.append(full_fig)
-        axes.append(full_axes)
+        if model['plotting']['quick_plot']:
+            quick_fig, quick_axes = make_figs.make_quick_fig(u=u, v=v, vis=vis,
+                                                             weights=weights, sol=sol,
+                                                             bin_widths=model['plotting']['bin_widths'],
+                                                             priors=priors,
+                                                             dist=model['plotting']['distance'],
+                                                             logx=model['plotting']['plot_in_logx'],
+                                                             force_style=model['plotting']['force_style'],
+                                                             save_prefix=model['input_output']['save_prefix'],
+                                                             stretch=model['plotting']['stretch'],
+                                                             gamma=model['plotting']['gamma'],
+                                                             asinh_a=model['plotting']['asinh_a']
+                                                             )
 
-    if model['plotting']['diag_plot']:
-        diag_fig, diag_axes, _ = make_figs.make_diag_fig(sol.r, sol.q,
-                                                         iteration_diagnostics,
-                                                         model['plotting']['iter_plot_range'],
-                                                         model['plotting']['force_style'],
-                                                         model['input_output']['save_prefix']
-                                                         )
+            figs.append(quick_fig)
+            axes.append(quick_axes)
 
-        figs.append(diag_fig)
-        axes.append(diag_axes)
+        if model['plotting']['full_plot']:
+            full_fig, full_axes = make_figs.make_full_fig(u=u, v=v, vis=vis,
+                                                          weights=weights, sol=sol,
+                                                          bin_widths=model['plotting']['bin_widths'],
+                                                          priors=priors,
+                                                          dist=model['plotting']['distance'],
+                                                          logx=model['plotting']['plot_in_logx'],
+                                                          force_style=model['plotting']['force_style'],
+                                                          save_prefix=model['input_output']['save_prefix'],
+                                                          norm_residuals=model['plotting']['norm_residuals'],
+                                                          stretch=model['plotting']['stretch'],
+                                                          gamma=model['plotting']['gamma'],
+                                                          asinh_a=model['plotting']['asinh_a']
+                                                          )
 
-    if model['analysis']['compare_profile']:
-        dat = np.genfromtxt(model['analysis']['compare_profile']).T
+            figs.append(full_fig)
+            axes.append(full_axes)
 
-        if len(dat) not in [2,3,4]:
-            raise ValueError("The file in your .json's `analysis` --> "
-                             "`compare_profile` must have 2, 3 or 4 "
-                             "columns: r [arcsec], I [Jy / sr], "
-                             "negative uncertainty [Jy / sr] (optional), "
-                             "positive uncertainty [Jy / sr] (optional, "
-                             "assumed equal to negative uncertainty if not "
-                             "provided).")
+        if model['plotting']['diag_plot']:
+            diag_fig, diag_axes, _ = make_figs.make_diag_fig(r=sol.r, q=sol.q,
+                                                             iteration_diagnostics=iteration_diagnostics,
+                                                             iter_plot_range=model['plotting']['iter_plot_range'],
+                                                             force_style=model['plotting']['force_style'],
+                                                             save_prefix=model['input_output']['save_prefix']
+                                                             )
 
-        r_clean, I_clean = dat[0], dat[1]
-        if len(dat) == 3:
-            lo_err_clean, hi_err_clean = dat[2], dat[2]
-        elif len(dat) == 4:
-            lo_err_clean, hi_err_clean = dat[2], dat[3]
-        else:
-            lo_err_clean, hi_err_clean = None, None
-        clean_profile = {'r': r_clean, 'I': I_clean, 'lo_err': lo_err_clean,
-                         'hi_err': hi_err_clean}
+            figs.append(diag_fig)
+            axes.append(diag_axes)
 
-        mean_convolved = None
-        if model['analysis']['clean_beam']['bmaj'] is not None:
-            mean_convolved = utilities.convolve_profile(sol.r, sol.mean,
-                                                        geom.inc, geom.PA,
-                                                        model['analysis']['clean_beam'])
+        if model['analysis']['compare_profile']:
+            dat = np.genfromtxt(model['analysis']['compare_profile']).T
 
-        clean_fig, clean_axes = make_figs.make_clean_comparison_fig(u, v, vis,
-                                                                    weights, sol,
-                                                                    clean_profile,
-                                                                    model['plotting']['bin_widths'],
-                                                                    model['plotting']['gamma'],
-                                                                    mean_convolved,
-                                                                    model['plotting']['distance'],
-                                                                    model['plotting']['force_style'],
-                                                                    model['input_output']['save_prefix']
-                                                                    )
+            if len(dat) not in [2,3,4]:
+                raise ValueError("The file in your .json's `analysis` --> "
+                                 "`compare_profile` must have 2, 3 or 4 "
+                                 "columns: r [arcsec], I [Jy / sr], "
+                                 "negative uncertainty [Jy / sr] (optional), "
+                                 "positive uncertainty [Jy / sr] (optional, "
+                                 "assumed equal to negative uncertainty if not "
+                                 "provided).")
 
-        figs.append(clean_fig)
-        axes.append(clean_axes)
+            r_clean, I_clean = dat[0], dat[1]
+            if len(dat) == 3:
+                lo_err_clean, hi_err_clean = dat[2], dat[2]
+            elif len(dat) == 4:
+                lo_err_clean, hi_err_clean = dat[2], dat[3]
+            else:
+                lo_err_clean, hi_err_clean = None, None
+            clean_profile = {'r': r_clean, 'I': I_clean, 'lo_err': lo_err_clean,
+                             'hi_err': hi_err_clean}
 
-    io.save_fit(u, v, vis, weights, sol,
-                model['input_output']['save_prefix'],
-                model['input_output']['save_solution'],
-                model['input_output']['save_profile_fit'],
-                model['input_output']['save_vis_fit'],
-                model['input_output']['save_uvtables'],
-                model['input_output']['iteration_diag'],
-                iteration_diagnostics,
-                model['input_output']['format']
-                )
+            mean_convolved = None
+            if model['analysis']['clean_beam']['bmaj'] is not None:
+                mean_convolved = utilities.convolve_profile(sol.r, sol.mean,
+                                                            geom.inc, geom.PA,
+                                                            model['analysis']['clean_beam'])
+
+            clean_fig, clean_axes = make_figs.make_clean_comparison_fig(u=u, v=v, vis=vis,
+                                                                        weights=weights, sol=sol,
+                                                                        clean_profile=clean_profile,
+                                                                        bin_widths=model['plotting']['bin_widths'],
+                                                                        stretch=model['plotting']['stretch'],
+                                                                        gamma=model['plotting']['gamma'],
+                                                                        asinh_a=model['plotting']['asinh_a'],
+                                                                        mean_convolved=mean_convolved,
+                                                                        dist=model['plotting']['distance'],
+                                                                        force_style=model['plotting']['force_style'],
+                                                                        save_prefix=model['input_output']['save_prefix']
+                                                                        )
+
+            figs.append(clean_fig)
+            axes.append(clean_axes)
 
     return figs, axes, model
 
@@ -635,8 +691,8 @@ def perform_bootstrap(u, v, vis, weights, geom, model):
         The axes of the produced figure
     """
 
-    if (type(model['hyperparameters']['alpha']) or \
-    type(model['hyperparameters']['wsmooth'])) is list:
+    if type(model['hyperparameters']['alpha']) is list or \
+    type(model['hyperparameters']['wsmooth']) is list:
         raise ValueError("For the bootstrap, both `alpha` and `wsmooth` in your "
                          "parameter file must be a float, not a list.")
 
@@ -655,7 +711,7 @@ def perform_bootstrap(u, v, vis, weights, geom, model):
         u_s, v_s, vis_s, w_s = utilities.draw_bootstrap_sample(
             u, v, vis, weights)
 
-        sol, _ = perform_fit(u_s, v_s, vis_s, w_s, geom, model)
+        sol, iteration_diagnostics = perform_fit(u_s, v_s, vis_s, w_s, geom, model)
 
         if model['hyperparameters']['nonnegative']:
             profiles_bootstrap.append(sol._nonneg)
@@ -669,10 +725,10 @@ def perform_bootstrap(u, v, vis, weights, geom, model):
 
     np.savez(bootstrap_path, r=sol.r, profiles=np.array(profiles_bootstrap))
 
-    boot_fig, boot_axes = make_figs.make_bootstrap_fig(sol.r,
-                                                       profiles_bootstrap,
-                                                       model['plotting']['force_style'],
-                                                       model['input_output']['save_prefix']
+    boot_fig, boot_axes = make_figs.make_bootstrap_fig(r=sol.r,
+                                                       profiles=profiles_bootstrap,
+                                                       force_style=model['plotting']['force_style'],
+                                                       save_prefix=model['input_output']['save_prefix']
                                                        )
 
     return boot_fig, boot_axes
@@ -694,7 +750,8 @@ def main(*args):
     geom = determine_geometry(u, v, vis, weights, model)
 
     if model['modify_data']['baseline_range'] or \
-            model['modify_data']['correct_weights']:
+            model['modify_data']['correct_weights'] or \
+            model['modify_data']['norm_wle']:
         u, v, vis, weights = alter_data(
             u, v, vis, weights, geom, model)
 
@@ -704,8 +761,8 @@ def main(*args):
 
         return boot_fig, boot_axes
 
-    elif (type(model['hyperparameters']['alpha']) or \
-    type(model['hyperparameters']['wsmooth'])) is list:
+    elif type(model['hyperparameters']['alpha']) is list or \
+    type(model['hyperparameters']['wsmooth']) is list:
         multifit_fig, multifit_axes = run_multiple_fits(u, v, vis, weights,
                                                         geom, model)
 
