@@ -23,6 +23,8 @@ import logging
 import numpy as np
 from scipy.interpolate import interp1d
 
+from frank.constants import deg_to_rad, sterad_to_arsec
+
 def arcsec_baseline(x):
     """
     Provide x as a radial scale [arcsec] to return the corresponding baseline
@@ -45,6 +47,61 @@ def arcsec_baseline(x):
     return converted
 
 
+def jy_convert(x, conversion, bmaj=None, bmin=None):
+    """
+    Provide x as a brightness in one of the units [Jy / beam], [Jy / arcsec^2],
+    [Jy / sterad] to convert x to another of these units
+
+    Parameters
+    ----------
+    x : float
+        Brightness in one of: [Jy / beam], [Jy / arcsec^2], [Jy / sterad]
+    conversion : { 'beam_sterad', 'beam_arcsec2', 'arcsec2_beam',
+                   'arcsec2_sterad', 'sterad_beam', 'sterad_arcsec2'}
+        The unit conversion to perform, e.g., 'beam_sterad' converts
+        [Jy / beam] to [Jy / sterad]
+    bmaj : float, optional
+        Beam FWHM along the major axis [arcsec]
+    bmin : float, optional
+        Beam FWHM along the minor axis [arcsec]
+
+    Returns
+    -------
+    converted : float
+        Brightness in converted units
+
+    """
+    if (bmaj is None or bmin is None) and conversion in ['beam_sterad',
+                                                       'beam_arcsec2',
+                                                       'arcsec2_beam',
+                                                       'sterad_beam']:
+        raise ValueError('bmaj and bmin must be specified to perform the'
+        ' conversion {}'.format(conversion))
+
+    if bmaj is not None and bmin is not None:
+        beam_solid_angle = np.pi * bmaj  * bmin / (4 * np.log(2))
+
+    if conversion == 'beam_arcsec2':
+        converted = x / beam_solid_angle
+    elif conversion == 'arcsec2_beam':
+        converted = x * beam_solid_angle
+    elif conversion == 'arcsec2_sterad':
+        converted = x * sterad_to_arsec
+    elif conversion == 'sterad_arcsec2':
+        converted = x / sterad_to_arsec
+    elif conversion == 'beam_sterad':
+        converted = x / beam_solid_angle * sterad_to_arsec
+    elif conversion == 'sterad_beam':
+        converted = x * beam_solid_angle / sterad_to_arsec
+    else:
+        raise AttributeError("conversion must be one of {}"
+                             "".format(['beam_sterad', 'beam_arcsec',
+                             'arcsec_beam', 'arcsec_sterad',
+                             'sterad_beam', 'sterad_arcsec']))
+
+    return converted
+
+
 class UVDataBinner(object):
     r"""
     Average uv-data into bins of equal size.
@@ -56,7 +113,8 @@ class UVDataBinner(object):
     uv : array, unit = :math:`\lambda`
         Baselines of the data to bin
     V : array, unit = Jy
-        Complex visibilities
+        Observed visibility. If complex, both the real and imaginary
+        components will be binned. Else only the real part will be binned.
     weights : array, unit = Jy^-2
         Weights on the visibility points
     bin_width : float, unit = :math:`\lambda`
@@ -107,19 +165,24 @@ class UVDataBinner(object):
         mu = self._V[self.determine_uv_bin(uv)]
 
         #   2) Compute weighted error for bins with n > 1
-        err = self.bin_quantities(uv, weights**2,
-                                  (V-mu).real**2 + 1j * (V-mu).imag**2)
+        quantities = (V-mu).real**2
+        if np.iscomplexobj(V):
+            quantities = quantities + 1j * (V-mu).imag**2
+        err = self.bin_quantities(uv, weights**2, quantities)
 
         idx2 = bin_n > 1
         err[idx2] /= bin_wgt[idx2]**2 * (1 - 1 / bin_n[idx2])
 
-        bin_vis_err[idx2] = \
-            np.sqrt(err.real[idx2]) + 1.j * np.sqrt(err.imag[idx2])
+        temp_error = np.sqrt(err.real[idx2])
+        if np.iscomplexobj(V):
+            temp_error = temp_error + 1.j * np.sqrt(err.imag[idx2])
+        bin_vis_err[idx2] = temp_error
 
         #   3) Use a sensible error for bins with one baseline
         idx1 = bin_n == 1
-        bin_vis_err[idx1].real = bin_vis_err[idx1].imag = \
-            1 / np.sqrt(bin_wgt[idx1])
+        bin_vis_err[idx1].real = 1 / np.sqrt(bin_wgt[idx1])
+        if np.iscomplexobj(V):
+            bin_vis_err[idx1].imag = bin_vis_err[idx1].real
         bin_vis_err[mask] = np.nan
 
         #   4) Store the error
@@ -281,6 +344,9 @@ def normalize_uv(u, v, wle):
 
     """
 
+    logging.info('  Normalizing u and v coordinates by provided'
+                 ' observing wavelength of {} m'.format(wle))
+
     u_normed = u / wle
     v_normed = v / wle
 
@@ -338,23 +404,25 @@ def cut_data_by_baseline(u, v, vis, weights, cut_range, geometry=None):
 
     return u_cut, v_cut, vis_cut, weights_cut
 
-def estimate_weights(u, v, V, nbins=300, log=True, use_median=False):
+def estimate_weights(u, v=None, V=None, nbins=300, log=True, use_median=False):
     r"""
     Estimate the weights using the variance of the binned visibilities.
 
     The estimation is done assuming that the variation in each bin is dominated
     by the noise. This will be true if:
-        1) The source is axi-symmetric,
-        2) The uv-points have been deprojected,
-        3) The bins are not too wide,
+    1) The source is axi-symmetric,
+    2) The uv-points have been deprojected,
+    3) The bins are not too wide,
     Otherwise the variance may be dominated by intrinsic variations in the
     visibilities.
 
     Parameters
     ----------
     u, v : array, unit = :math:`\lambda`
-        u and v coordinates of observations (deprojected).
-    V : array, unit = Jy
+        u and v coordinates of observations (deprojected). Data will be binned
+        by baseline. If v is not None, np.hpot(u,v) will be used instead. Note
+        that if V is None the argument v will be intepreted as V instead
+    V : array, unit = Jy, default = None
         Observed visibility. If complex, the weights will be computed from the
         average of the variance of the real and imaginary components, as in
         CASA's statwt. Otherwise the variance of the real part is used.
@@ -379,11 +447,30 @@ def estimate_weights(u, v, V, nbins=300, log=True, use_median=False):
         - Bins with only one uv point do not have a variance estimate. Thus
           the mean of the variance in the two adjacent bins is used instead.
 
+    Examples
+    --------
+        All of the following calls will work as expected:
+            `estimate_weights(u, v, V) `
+            `estimate_weights(u, V)`
+            `estimate_weights(u, V=V)`
+        In each case the variance of V in the uv-bins is used to estimate the
+        weights. The first call will use q = np.hypot(u, v) in the uv-bins. The
+        second and third calls are equivalent to the first with u=0.
     """
 
-    logging.info('  Estimating visibility weights.')
+    logging.info('  Estimating visibility weights')
+
+    if V is None:
+        if v is not None:
+            V = v
+            q = np.abs(u)
+        else:
+            raise ValueError("The visibilities, V, must be supplied")
+    elif v is not None:
+        q = np.hypot(u,v)
+    else:
+        q = np.abs(u)
 
-    q = np.hypot(u,v)
     if log:
         q = np.log(q)
         q -= q.min()
@@ -402,13 +489,16 @@ def estimate_weights(u, v, V, nbins=300, log=True, use_median=False):
         var = uvBin.error.real**2 * uvBin.bin_counts
 
     if use_median:
-        return np.full(len(u), 1/np.median(var[uvBin.bin_counts > 1]))
+        logging.info('    Setting all weights as median binned visibility variance')
+        return np.full(len(u), 1/np.ma.median(var[uvBin.bin_counts > 1]))
     else:
+        logging.info('    Setting weights according to baseline-dependent binned'
+                     ' visibility variance')
         # For bins with 1 uv point, use the average of the adjacent bins
-        no_var = np.argwhere(uvBin.bin_counts == 1)
+        no_var = np.argwhere(uvBin.bin_counts == 1).reshape(-1)
         if len(no_var) > 0:
             # Find the location `loc` of the bad points in the array of good points
-            good_var = np.argwhere(uvBin.bin_counts > 1)
+            good_var = np.argwhere(uvBin.bin_counts > 1).reshape(-1)
             loc = np.searchsorted(good_var, no_var, side='right')
 
             # Set the variance to the average of the two adjacent bins
@@ -458,38 +548,85 @@ def draw_bootstrap_sample(u, v, vis, weights):
     return u_boot, v_boot, vis_boot, weights_boot
 
 
-def sweep_profile(r, I, axis=0):
+def sweep_profile(r, I, project=False, phase_shift=False, geom=None, axis=0,
+                  xmax=None, ymax=None, dr=None):
     r"""
     Sweep a 1D radial brightness profile over :math:`2 \pi` to yield a 2D
-    brightness distribution
+    brightness distribution. Optionally project this sweep by a supplied
+    geometry.
 
     Parameters
     ----------
     r : array
-          Radial coordinates at which the 1D brightness profile is defined
+        Radial coordinates at which the 1D brightness profile is defined
     I : array
-          Brightness values at r
+        Brightness values at r
+    project : bool, default = False
+        Whether to project the swept profile by the supplied geom
+    phase_shift : bool, default = False
+        Whether to phase shift the projected profile by the supplied geom.
+        If False, the source will be centered in the image
+    geom : SourceGeometry object, default=None
+        Fitted geometry (see frank.geometry.SourceGeometry). Here we use
+        geom.inc [deg], geom.PA [deg], geom.dRA [arcsec], geom.dDec [arcsec] if
+        project=True
     axis : int, default = 0
-          Axis over which to interpolate the 1D profile
+        Axis over which to interpolate the 1D profile
+    xmax, ymax : float, optional, default = None
+        Value setting the x- and y-bounds of the image (same units as r). The
+        positive and negative bounds are both set to this value (modulo sign).
+        If not provided, these will be set to r.max()
+    dr : float, optional, default = None
+        Pixel size (same units as r). If not provided, it will be set at the
+        same spatial scale as r
 
     Returns
     -------
     I2D : array, shape = (len(r), len(r))
-        2D brightness distribution
+        2D brightness distribution (projected if project=True)
     xmax : float
         Maximum x-value of the 2D grid
     ymax : float
         Maximum y-value of the 2D grid
 
+    Notes
+    -----
+    Sign convention: a negative geom.dRA shifts the source to the right
+    in the image
     """
+    if project:
+        inc, pa, dra, ddec = geom.inc, geom.PA, geom.dRA, geom.dDec
+        inc *= deg_to_rad
+        pa *= deg_to_rad
+        if not phase_shift:
+            dra, ddec = 0., 0.
+
+        cos_i = np.cos(inc)
+        cos_pa, sin_pa = np.cos(pa), np.sin(pa)
+
+    if xmax is None:
+        xmax = r.max()
+    if ymax is None:
+        ymax = r.max()
+
+    if dr is None:
+        dr = np.mean(np.diff(r))
 
-    xmax = ymax = r.max()
-    dr = np.mean(np.diff(r))
     x = np.linspace(-xmax, xmax, int(xmax/dr) + 1)
     y = np.linspace(-ymax, ymax, int(ymax/dr) + 1)
 
-    xi, yi = np.meshgrid(x, y)
-    r1D = np.hypot(xi, yi)
+    if phase_shift:
+        xi, yi = np.meshgrid(x + dra, y - ddec)
+    else:
+        xi, yi = np.meshgrid(x, y)
+
+    if project:
+        xp  = xi * cos_pa + yi * sin_pa
+        yp  = -xi * sin_pa + yi * cos_pa
+        xp /= cos_i
+        r1D = np.hypot(xp, yp)
+    else:
+        r1D = np.hypot(xi, yi)
 
     im_shape = r1D.shape + I.shape[1:]
 
@@ -519,7 +656,7 @@ def convolve_profile(r, I, disc_i, disc_pa, clean_beam,
         Dictionary with beam `bmaj` (FWHM of beam along its major axis) [arcsec],
         `bmin` (FWHM of beam along its minor axis) [arcsec],
         `pa` (beam position angle) [deg]
-    n_per_sigma : int, default = 5
+    n_per_sigma : int, default = 50
         Number of points per standard deviation of the Gaussian kernel (used
         for gridding)
     axis : int, default = 0
@@ -546,7 +683,7 @@ def convolve_profile(r, I, disc_i, disc_pa, clean_beam,
 
     PA = (disc_pa - clean_beam['beam_pa']) * np.pi / 180.
 
-    cos_i = np.cos(disc_i) * np.pi/180.
+    cos_i = np.cos(disc_i * np.pi/180.)
     cos_PA = np.cos(PA)
     sin_PA = np.sin(PA)
 
